#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


import os
import json
from dataclasses import dataclass, field

from mephisto.data_model.requester import Requester
from mephisto.data_model.constants.assignment_state import AssignmentState
from mephisto.data_model._db_backed_meta import (
    MephistoDBBackedMeta,
    MephistoDataModelComponentMixin,
)
from mephisto.utils.dirs import get_dir_for_run

from omegaconf import OmegaConf, MISSING

from typing import List, Optional, Dict, Mapping, TYPE_CHECKING, Any

if TYPE_CHECKING:
    from mephisto.abstractions.database import MephistoDB
    from mephisto.data_model.assignment import Assignment
    from mephisto.abstractions.blueprint import Blueprint, SharedTaskState
    from mephisto.data_model.worker import Worker
    from mephisto.data_model.unit import Unit
    from mephisto.abstractions.crowd_provider import CrowdProvider
    from mephisto.data_model.task import Task
    from omegaconf import DictConfig

from mephisto.utils.logger_core import get_logger

logger = get_logger(name=__name__)


@dataclass
class TaskRunArgs:
    """Object for grouping the contents to configure a class"""

    task_name: Optional[str] = field(
        default=MISSING,
        metadata={
            "help": "Grouping to launch this task run under, none defaults to the blueprint type"
        },
    )
    task_title: str = field(
        default=MISSING,
        metadata={
            "help": "Display title for your task on the crowd provider.",
            "required": True,
        },
    )
    task_description: str = field(
        default=MISSING,
        metadata={
            "help": "Longer form description for what your task entails.",
            "required": True,
        },
    )
    task_reward: float = field(
        default=MISSING,
        metadata={
            "help": "Amount to pay per worker per unit, in dollars.",
            "required": True,
        },
    )
    task_tags: str = field(
        default=MISSING,
        metadata={
            "help": "Comma seperated tags for workers to use to find your task.",
            "required": True,
        },
    )
    task_lifetime_in_seconds: int = field(
        default=60 * 60 * 24 * 31,
        metadata={"help": "The time that the task will last for before expiring"},
    )
    assignment_duration_in_seconds: int = field(
        default=30 * 60,
        metadata={"help": "Time that workers have to work on your task once accepted."},
    )
    no_submission_patience: int = field(
        default=60 * 60 * 12,
        metadata={
            "help": (
                "How long to wait between task submissions before shutting the run down "
                "for a presumed issue. Value in seconds, default 12 hours. "
            )
        },
    )
    allowed_concurrent: int = field(
        default=0,
        metadata={
            "help": "Maximum units a worker is allowed to work on at once. (0 is infinite)",
            "required": True,
        },
    )
    maximum_units_per_worker: int = field(
        default=0,
        metadata={
            "help": (
                "Maximum tasks of this task name that a worker can work on across all "
                "tasks that share this task_name. (0 is infinite)"
            )
        },
    )
    max_num_concurrent_units: int = field(
        default=0,
        metadata={
            "help": (
                "Maximum units that will be released simultaneously, setting a limit "
                "on concurrent connections to Mephisto overall. (0 is infinite)"
            )
        },
    )
    submission_timeout: int = field(
        default=600,
        metadata={
            "help": (
                "Time that mephisto will wait after marking a task done before abandoning "
                "waiting for the worker to actually press submit."
            )
        },
    )

    post_install_script: str = field(
        default="",
        metadata={
            "help": (
                "The name of a shell script in your webapp directory that will run right after npm install and before npm build."
                "This can be useful for local package development where you would want to link a package after installing dependencies from package.json"
            )
        },
    )

    force_rebuild: bool = field(
        default=False,
        metadata={
            "help": (
                "Determines if npm build should be ran every time the task is ran."
                "By default there is an optimization that only builds the webapp when there is a change in its contents."
                "It would make sense to set this to true when doing local package development as you want to force a rebuild after running the post_install_script."
            )
        },
    )

    @classmethod
    def get_mock_params(cls) -> str:
        """Returns a param string with default / mock arguments to use for testing"""
        from mephisto.operations.hydra_config import MephistoConfig

        return OmegaConf.structured(
            MephistoConfig(
                task=TaskRunArgs(
                    task_title="Mock Task Title",
                    task_reward=0.3,
                    task_tags="mock,task,tags",
                    task_description="This is a test description",
                )
            )
        )


class TaskRun(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedMeta):
    """
    This class tracks an individual run of a specific task, and handles state management
    for the set of assignments within
    """

    ArgsClass = TaskRunArgs

    def __init__(
        self,
        db: "MephistoDB",
        db_id: str,
        row: Optional[Mapping[str, Any]] = None,
        _used_new_call: bool = False,
    ):
        if not _used_new_call:
            raise AssertionError(
                "Direct TaskRun and data model access via TaskRun(db, id) is "
                "now deprecated in favor of calling TaskRun.get(db, id). "
            )
        self.db: "MephistoDB" = db
        if row is None:
            row = db.get_task_run(db_id)
        assert row is not None, f"Given db_id {db_id} did not exist in given db"
        self.db_id: str = row["task_run_id"]
        self.task_id: str = row["task_id"]
        self.requester_id: str = row["requester_id"]
        self.param_string: str = row["init_params"]
        try:
            self.args: "DictConfig" = OmegaConf.create(json.loads(self.param_string))
        except Exception as e:
            self.args = OmegaConf.create({})
            print(e)
        self.start_time = row["creation_date"]
        self.provider_type: str = row["provider_type"]
        self.task_type: str = row["task_type"]
        self.sandbox: bool = row["sandbox"]
        self.assignments_generator_done: bool = False

        # properties with deferred loading
        self.__is_completed = row["is_completed"]
        self.__has_assignments = False
        self.__task: Optional["Task"] = None
        self.__requester: Optional["Requester"] = None
        self.__run_dir: Optional[str] = None
        self.__blueprint: Optional["Blueprint"] = None
        self.__crowd_provider: Optional["CrowdProvider"] = None

    def get_units(self) -> List["Unit"]:
        """
        Return the units associated with this task run.
        """
        return self.db.find_units(task_run_id=self.db_id)

    def get_valid_units_for_worker(self, worker: "Worker") -> List["Unit"]:
        """
        Get any units that the given worker could work on in this
        task run
        """
        config = self.get_task_args()

        # TODO(#773) handle with temporary local qualifications to allow
        # pushing real exclusionary qualifications after exceeding
        # maximum_units_per_worker
        if config.allowed_concurrent != 0 or config.maximum_units_per_worker:
            current_units = self.db.find_units(
                task_run_id=self.db_id,
                worker_id=worker.db_id,
                status=AssignmentState.ASSIGNED,
            )
            currently_active = len(current_units)
            if config.allowed_concurrent != 0:
                if currently_active >= config.allowed_concurrent:
                    logger.debug(
                        f"{worker} at maximum concurrent units {currently_active}"
                    )
                    return []  # currently at the maximum number of concurrent units
            if config.maximum_units_per_worker != 0:
                completed_types = AssignmentState.completed()
                related_units = self.db.find_units(
                    task_id=self.task_id,
                    worker_id=worker.db_id,
                )
                currently_completed = len(
                    [u for u in related_units if u.db_status in completed_types]
                )
                if (
                    currently_active + currently_completed
                    >= config.maximum_units_per_worker
                ):
                    logger.debug(
                        f"{worker} at maximum units {currently_active}, {currently_completed}"
                    )
                    return []  # Currently at the maximum number of units for this task

        task_units: List["Unit"] = self.get_units()
        unit_assigns: Dict[str, List["Unit"]] = {}
        for unit in task_units:
            assignment_id = unit.assignment_id
            if assignment_id not in unit_assigns:
                unit_assigns[assignment_id] = []
            unit_assigns[assignment_id].append(unit)

        # Cannot pair with self
        units: List["Unit"] = []
        for unit_set in unit_assigns.values():
            is_self_set = map(lambda u: u.worker_id == worker.db_id, unit_set)
            if not any(is_self_set):
                units += unit_set

        # Valid units must be launched and must not be special units (negative indices)
        # Can use db_status directly rather than polling in the critical path, as in
        # the worst case we miss the transition from an active to launched unit
        valid_units = [
            u
            for u in units
            if u.db_status == AssignmentState.LAUNCHED and u.unit_index >= 0
        ]
        logger.debug(f"Found {len(valid_units)} available units")

        # Should load cached blueprint for SharedTaskState
        blueprint = self.get_blueprint()
        ret_units = [
            u
            for u in valid_units
            if blueprint.shared_state.worker_can_do_unit(worker, u)
        ]

        logger.debug(f"This worker is qualified for {len(ret_units)} unit.")
        logger.debug(f"Found {ret_units[:3]} for {worker}.")
        return ret_units

    def clear_reservation(self, unit: "Unit") -> None:
        """
        Remove the holder used to reserve a unit
        """
        file_name = f"unit_res_{unit.db_id}"
        write_dir = os.path.join(self.get_run_dir(), "reservations")
        if os.path.exists(os.path.join(write_dir, file_name)):
            os.unlink(os.path.join(write_dir, file_name))
            logger.debug(f"Cleared reservation {file_name} for {unit}")

    def reserve_unit(self, unit: "Unit") -> Optional["Unit"]:
        """
        'Atomically' reserve a unit by writing to the filesystem. If
        the file creation fails, return none
        """
        file_name = f"unit_res_{unit.db_id}"
        write_dir = os.path.join(self.get_run_dir(), "reservations")
        os.makedirs(write_dir, exist_ok=True)
        try:
            with open(os.path.join(write_dir, file_name), "x") as res_file:
                pass  # Creating the file is sufficient
        except FileExistsError:
            print(os.path.join(write_dir, file_name), " existed")
            return None
        logger.debug(f"Reserved {unit} with {file_name}")
        return unit

    def get_blueprint(
        self,
        args: Optional["DictConfig"] = None,
        shared_state: Optional["SharedTaskState"] = None,
    ) -> "Blueprint":
        """Return the runner associated with this task run"""
        from mephisto.operations.registry import get_blueprint_from_type
        from mephisto.abstractions.blueprint import SharedTaskState

        if self.__blueprint is None:
            cache = False
            if args is None:
                args = self.args
            else:
                cache = True

            BlueprintClass = get_blueprint_from_type(self.task_type)
            if shared_state is None:
                shared_state = BlueprintClass.SharedStateClass()
            if not cache:
                return BlueprintClass(self, args, shared_state)
            self.__blueprint = BlueprintClass(self, args, shared_state)
        return self.__blueprint

    def get_provider(self) -> "CrowdProvider":
        """Return the crowd provider used to launch this task"""
        from mephisto.operations.registry import get_crowd_provider_from_type

        if self.__crowd_provider is None:
            CrowdProviderClass = get_crowd_provider_from_type(self.provider_type)
            self.__crowd_provider = CrowdProviderClass(self.db)
        return self.__crowd_provider

    def get_task(self) -> "Task":
        """Return the task used to initialize this run"""
        if self.__task is None:
            from mephisto.data_model.task import Task

            self.__task = Task.get(self.db, self.task_id)
        return self.__task

    def get_task_args(self) -> "DictConfig":
        return self.args.task

    def get_requester(self) -> Requester:
        """
        Return the requester that started this task.
        """
        if self.__requester is None:
            self.__requester = Requester.get(self.db, self.requester_id)
        return self.__requester

    def get_has_assignments(self) -> bool:
        """See if this task run has any assignments launched yet"""
        if not self.__has_assignments:
            if len(self.get_assignments()) > 0:
                self.__has_assignments = True
        return self.__has_assignments

    def get_assignments(self, status: Optional[str] = None) -> List["Assignment"]:
        """
        Get assignments for this run, optionally filtering by their
        current status
        """
        assert (
            status is None or status in AssignmentState.valid()
        ), "Invalid assignment status"
        assignments = self.db.find_assignments(task_run_id=self.db_id)
        if status is not None:
            assignments = [a for a in assignments if a.get_status() == status]
        return assignments

    def get_assignment_statuses(self) -> Dict[str, int]:
        """
        Get the statistics for all of the assignments for this run.
        """
        assigns = self.get_assignments()
        assigns_with_status = [(x, x.get_status()) for x in assigns]
        return {
            status: len(
                [x for x, had_status in assigns_with_status if had_status == status]
            )
            for status in AssignmentState.valid()
        }

    def update_completion_progress(self, task_launcher=None, status=None) -> None:
        """Flag the task run that the assignments' generator has finished"""
        if task_launcher:
            if task_launcher.get_assignments_are_all_created():
                self.assignments_generator_done = True
        if status:
            self.assignments_generator_done = status

    def get_is_completed(self) -> bool:
        """get the completion status of this task"""
        self.sync_completion_status()
        return self.__is_completed

    def sync_completion_status(self) -> None:
        """
        Update the is_complete status for this task run based on completion
        of subassignments. If this task run has no subassignments yet, it
        is not complete
        """
        if not self.__is_completed and self.get_has_assignments():
            statuses = self.get_assignment_statuses()
            has_incomplete = False
            for status in AssignmentState.incomplete():
                if statuses[status] > 0:
                    has_incomplete = True
            if not has_incomplete and self.assignments_generator_done is not False:
                self.db.update_task_run(self.db_id, is_completed=True)
                self.__is_completed = True

    def get_run_dir(self) -> str:
        """
        Return the directory where the data from this run is stored
        """
        if self.__run_dir is None:
            task = self.get_task()
            project = task.get_project()
            if project is None:
                self.__run_dir = get_dir_for_run(self)
            else:
                self.__run_dir = get_dir_for_run(self, project.project_name)
            os.makedirs(self.__run_dir, exist_ok=True)
        return self.__run_dir

    def get_total_spend(self) -> float:
        """
        Return the total amount spent on this run, based on any assignments
        that are still in a payable state.
        """
        assigns = self.get_assignments()
        total_amount = 0.0
        for assign in assigns:
            total_amount += assign.get_cost_of_statuses(AssignmentState.payable())
        return total_amount

    def to_dict(self) -> Dict[str, Any]:
        """Return a dict containing any important information about this task run"""
        return {
            "task_run_id": self.db_id,
            "task_id": self.task_id,
            "task_name": self.get_task().task_name,
            "task_type": self.task_type,
            "start_time": self.start_time,
            "params": OmegaConf.to_container(self.args),
            "param_string": self.param_string,
            "task_status": self.get_assignment_statuses(),
            "sandbox": self.get_requester().is_sandbox(),
        }

    def __repr__(self) -> str:
        return f"TaskRun({self.db_id})"

    @staticmethod
    def new(
        db: "MephistoDB", task: "Task", requester: Requester, param_string: str
    ) -> "TaskRun":
        """
        Create a new run for the given task with the given params
        """
        db_id = db.new_task_run(
            task.db_id,
            requester.db_id,
            param_string,
            requester.provider_type,
            task.task_type,
        )
        task_run = TaskRun.get(db, db_id)
        logger.debug(f"Created new task run {task_run}")
        return task_run



"""
Script that allows for accepted tips to be removed.
Accepted tips are collected by retrieving all units and then getting the agent of each unit.
The tips for each agent state is filtered to only collect accepted tips.

For each accepted tip you have the option to remove the tip from the agent state.
"""

import enum
from genericpath import exists
from typing import Any, Dict, List
import csv
from mephisto.abstractions.databases.local_database import LocalMephistoDB
from mephisto.data_model.task_run import TaskRun
from mephisto.scripts.local_db.review_tips_for_task import (
    remove_tip_from_metadata,
)
from mephisto.tools.data_browser import DataBrowser as MephistoDataBrowser
from mephisto.tools.scripts import print_out_task_names
from mephisto.utils.rich import console
from rich import print
from rich.prompt import Prompt
from rich.table import Table, Column
from rich import box


class TipsRemovalType(enum.Enum):
    REMOVE = "y"
    KEEP = "n"


def remove_tip_from_tips_file(
    tips: List[Dict[str, Any]],
    i: int,
    task_run: TaskRun,
):
    """Removes the specified tip row from the csv file"""
    tip_id = tips[i]["id"]
    blueprint_task_run_args = task_run.args["blueprint"]
    if "tips_location" in blueprint_task_run_args:
        tips_location = blueprint_task_run_args["tips_location"]
        does_file_exist = exists(tips_location)
        if does_file_exist == False:
            print(
                "\n[red]You do not have a tips.csv file in your task's output directory[/red]"
            )
            quit()

        lines_to_write = []
        # Getting the row in the csv file
        with open(tips_location) as read_tips_file:
            reader = csv.reader(read_tips_file)
            for row in reader:
                if row[0] != tip_id:
                    lines_to_write.append(row)

        # Writing all rows except the row to remove
        with open(tips_location, "w") as write_tips_file:
            writer = csv.writer(write_tips_file)
            writer.writerows(lines_to_write)


def main():
    db = LocalMephistoDB()
    mephisto_data_browser = MephistoDataBrowser(db)
    task_names = mephisto_data_browser.get_task_name_list()
    print_out_task_names("Tips Removal", task_names)
    task_name = Prompt.ask(
        "\nEnter the name of the task that you want to review the tips of",
        choices=task_names,
        show_choices=False,
    ).strip()

    units = mephisto_data_browser.get_all_units_for_task_name(task_name)
    if len(units) == 0:
        print("[red]No units were received[/red]")
        quit()
    for unit in units:
        if unit.agent_id is not None:
            unit_data = mephisto_data_browser.get_data_from_unit(unit)
            tips = unit_data["tips"]
            if tips is not None:
                accepted_tips = list(filter(lambda tip: tip["accepted"] == True, tips))
                accepted_tips_copy = accepted_tips.copy()

                for i in range(len(accepted_tips)):
                    current_tip_table = Table(
                        "Property",
                        Column("Value"),
                        title="\nTip {current_tip} of {total_number_of_tips} From Agent {agent_id}".format(
                            current_tip=i + 1,
                            total_number_of_tips=len(accepted_tips),
                            agent_id=unit.agent_id,
                        ),
                        box=box.ROUNDED,
                        expand=True,
                        show_lines=True,
                    )
                    current_tip_table.add_row("Tip Id", accepted_tips[i]["id"])
                    current_tip_table.add_row("Tip Header", accepted_tips[i]["header"])
                    current_tip_table.add_row("Tip Text", accepted_tips[i]["text"])
                    console.print(current_tip_table)

                    removal_response = Prompt.ask(
                        "\nDo you want to remove this tip? (Default: n)",
                        choices=[tips_type.value for tips_type in TipsRemovalType],
                        default=TipsRemovalType.KEEP.value,
                        show_default=False,
                    ).strip()
                    print("")

                    if removal_response == TipsRemovalType.REMOVE.value:
                        remove_tip_from_tips_file(
                            accepted_tips_copy, i, unit.get_task_run()
                        )
                        remove_tip_from_metadata(
                            accepted_tips, accepted_tips_copy, i, unit
                        )
                        print("Removed tip\n")
                    elif removal_response == TipsRemovalType.KEEP.value:
                        print("Did not remove tip\n")
    print("There are no more tips to look at\n")


if __name__ == "__main__":
    main()



from mephisto.client.cli_commands import get_wut_arguments
from mdutils.mdutils import MdUtils
from copy import deepcopy
from mephisto.operations.registry import get_valid_blueprint_types


def create_blueprint_info(blueprint_file, arg_dict):
    blueprint_file.new_paragraph(arg_dict["desc"].strip())
    arg_keys = list(arg_dict["args"].keys())
    first_arg = list(arg_dict["args"].keys())[0]
    first_arg_keys = list(arg_dict["args"][first_arg].keys())

    list_of_strings = deepcopy(first_arg_keys)

    for cur_key in arg_keys:
        extension_list = []
        for first_arg_key in first_arg_keys:
            item_content = arg_dict["args"][cur_key][first_arg_key]
            item_to_append = (
                "".join(item_content.splitlines())
                if isinstance(item_content, str)
                else item_content
            )
            if (
                isinstance(item_to_append, str)
                and item_to_append.rfind("mephisto/") != -1
            ):
                item_to_append = item_to_append[
                    item_to_append.rfind("mephisto/") : len(item_to_append)
                ]

            extension_list.append(item_to_append)
        list_of_strings.extend(extension_list)

    blueprint_file.new_line()

    # Add 1 to rows to account for header row
    blueprint_file.new_table(
        rows=len(arg_keys) + 1,
        columns=len(first_arg_keys),
        text=list_of_strings,
        text_align="left",
    )


def main():
    blueprint_file = MdUtils(
        file_name="../../../../docs/web/docs/reference/blueprints.md",
    )
    blueprint_file.new_header(level=1, title="Blueprints")
    blueprint_file.new_paragraph(
        "The blueprints contain all of the related code required to set up a task run."
    )
    valid_blueprint_types = get_valid_blueprint_types()
    for blueprint_type in valid_blueprint_types:
        blueprint_file.new_header(level=2, title=blueprint_type.replace("_", " "))
        args = get_wut_arguments(
            ("blueprint={blueprint_name}".format(blueprint_name=blueprint_type),)
        )
        arg_dict = args[0]
        create_blueprint_info(blueprint_file, arg_dict)

    blueprint_file.create_md_file()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import logging
from logging.handlers import RotatingFileHandler
from typing import Any, List, Optional, Dict, Set
from rich.logging import RichHandler
from mephisto.utils.rich import console

loggers: Dict[str, logging.Logger] = {}
global_log_level = logging.INFO
_seen_logs: Set[str] = set()


def warn_once(msg: str) -> None:
    """
    Log a warning, but only once.

    :param str msg: Message to display
    """
    global _seen_logs
    if msg not in _seen_logs:
        _seen_logs.add(msg)
        logging.warn(msg)


def set_mephisto_log_level(verbose: Optional[bool] = None, level: Optional[str] = None):
    """
    Set the global level for Mephisto logging, from
    which all other classes will set their logging.

    Verbose sets an option between DEBUG and INFO, while level allows setting
    a specific level, and takes precedence.

    Calling this function will override the desired log levels from individual files,
    and if you want to enable a specific level for a specific logger, you'll need
    to get that logger from the loggers dict and call setLevel
    """
    global global_log_level

    if verbose is None and level is None:
        raise ValueError("Must provide one of verbose or level")

    if verbose is not None:
        global_log_level = logging.DEBUG if verbose else logging.INFO

    if level is not None:
        global_log_level = logging.getLevelName(level.upper())

    for logger in loggers.values():
        logger.setLevel(global_log_level)


def get_logger(
    name: str,
    verbose: Optional[bool] = None,
    log_file: Optional[str] = None,
    level: Optional[str] = None,
) -> logging.Logger:
    """
    Gets the logger corresponds to each module
            Parameters:
                    name (string): the module name (__name__).
                    verbose (bool): INFO level activated if True.
                    log_file (string): path for saving logs locally.
                    level (string): logging level. Values options: [info, debug, warning, error, critical].

            Returns:
                    logger (logging.Logger): the corresponding logger to the given module name.
    """
    LOGFORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    LOGFORMAT_RICH = "%(message)s"
    global loggers
    found_logger = loggers.get(name)
    if found_logger is not None:
        return found_logger
    else:

        level_dict = {
            "debug": logging.DEBUG,
            "info": logging.INFO,
            "warning": logging.WARNING,
            "error": logging.ERROR,
            "critical": logging.CRITICAL,
        }
        ch = RichHandler()
        ch.setFormatter(logging.Formatter(LOGFORMAT_RICH))
        logging_format: List[Any] = [ch]
        if log_file is not None:
            logging_format.append(RotatingFileHandler(log_file))
        logging.basicConfig(
            level=level_dict[level.lower()] if level is not None else logging.INFO,
            format=LOGFORMAT,
            handlers=logging_format,
        )

        logger = logging.getLogger(name)

        logger.addHandler(ch)

        if level is not None:
            logger.setLevel(level_dict[level.lower()])
        elif verbose is not None:
            logger.setLevel(logging.DEBUG if verbose else logging.INFO)
        else:
            logger.setLevel(global_log_level)

        loggers[name] = logger
        return logger


BOLD_RED = "\u001b[31;1m"
RESET = "\u001b[0m"


def format_loud(target_text: str):
    return f"{BOLD_RED}{target_text}{RESET}"



"""
Script that allows for tips to be approved or rejected.
Tips are collected by retrieving all units and then getting the agent of each unit.
The agent state has the tip data.

Approving a tip sets the accept property of the tip to be True.
This prevents it from appearing again when running this script.
It is stored in the agent state's metadata and in the assets/tips.csv file
in your task's directory.

Rejecting a tip deletes the tip from the tips list in the AgentState's metadata.
It also removed the row in the assets/tips.csv file in your task's directory.
"""
import csv
from genericpath import exists
from pathlib import Path
from typing import Any, List, Dict, Optional
from mephisto.abstractions.databases.local_database import LocalMephistoDB
from mephisto.data_model.agent import Agent
from mephisto.data_model.task_run import TaskRun
from mephisto.data_model.unit import Unit
from mephisto.data_model.worker import Worker
from mephisto.tools.data_browser import DataBrowser as MephistoDataBrowser
from rich import print
from rich import box
from rich.prompt import Prompt
from rich.prompt import FloatPrompt
from rich.table import Table
from mephisto.tools.scripts import print_out_task_names
from mephisto.utils.rich import console
import enum


class TipsReviewType(enum.Enum):
    ACCEPTED = "a"
    REJECTED = "r"
    SKIP = "s"


def get_index_of_value(lst: List[str], property: str) -> int:
    for i in range(len(lst)):
        if lst[i] == property:
            return i
    return 0


def add_row_to_tips_file(task_run: TaskRun, item_to_add: Dict[str, Any]):
    """Adds a row the tips csv file"""
    blueprint_task_run_args = task_run.args["blueprint"]
    if "tips_location" in blueprint_task_run_args:
        tips_location = blueprint_task_run_args["tips_location"]
        does_file_exist = exists(tips_location)
        if does_file_exist == False:
            # Creates the file
            create_tips_file = Path(tips_location)
            try:
                create_tips_file.touch(exist_ok=True)
            except FileNotFoundError:
                print(
                    "\n[red]Your task folder must have an assets folder in it.[/red]\n"
                )
                quit()

        with open(tips_location, "r") as inp, open(tips_location, "a+") as tips_file:
            field_names = list(item_to_add.keys())
            writer = csv.DictWriter(tips_file, fieldnames=field_names)
            reader = csv.reader(inp)
            # Add header if the file is newly created or empty
            if does_file_exist == False or len(list(reader)) == 0:
                writer.writeheader()
            writer.writerow(item_to_add)


def remove_tip_from_metadata(
    tips: List[Dict[str, Any]], tips_copy: List[Dict[str, Any]], i: int, unit: Unit
):
    """Removes a tip from metadata"""
    tips_id = [tip_obj["id"] for tip_obj in tips_copy]
    index_to_remove = get_index_of_value(tips_id, tips[i]["id"])
    assigned_agent: Optional[Agent] = unit.get_assigned_agent()

    if assigned_agent is not None:
        tips_copy.pop(index_to_remove)
        assigned_agent.state.update_metadata(
            property_name="tips", property_value=tips_copy
        )
    else:
        print("[red]An assigned agent was not able to be found for this tip[/red]")
        quit()


def accept_tip(tips: List, tips_copy: List, i: int, unit: Unit) -> None:
    """Accepts a tip in metadata"""
    tips_id = [tip_obj["id"] for tip_obj in tips_copy]
    # gets the index of the tip in the tip_copy list
    index_to_update = get_index_of_value(tips_id, tips[i]["id"])
    assigned_agent = unit.get_assigned_agent()

    if assigned_agent is not None:
        tips_copy[index_to_update]["accepted"] = True
        add_row_to_tips_file(unit.get_task_run(), tips_copy[index_to_update])
        assigned_agent.state.update_metadata(
            property_name="tips", property_value=tips_copy
        )


def main():
    db = LocalMephistoDB()
    mephisto_data_browser = MephistoDataBrowser(db)
    task_names = mephisto_data_browser.get_task_name_list()
    print_out_task_names("Tips Review", task_names)
    task_name = Prompt.ask(
        "\nEnter the name of the task that you want to review the tips of",
        choices=task_names,
        show_choices=False,
    ).strip()
    print("")
    units = mephisto_data_browser.get_all_units_for_task_name(task_name)
    if len(units) == 0:
        print("[red]No units were received[/red]")
        quit()
    for unit in units:
        if unit.agent_id is not None:
            unit_data = mephisto_data_browser.get_data_from_unit(unit)
            tips = unit_data["tips"]

            if tips is not None and len(tips) > 0:
                tips_copy = tips.copy()
                for i in range(len(tips)):
                    if tips[i]["accepted"] == False:
                        current_tip_table = Table(
                            "Property",
                            "Value",
                            title="\nTip {current_tip} of {total_number_of_tips} From Agent {agent_id}".format(
                                current_tip=i + 1,
                                total_number_of_tips=len(tips),
                                agent_id=unit.agent_id,
                            ),
                            box=box.ROUNDED,
                            expand=True,
                            show_lines=True,
                        )
                        current_tip_table.add_row("Tip Id", tips[i]["id"])
                        current_tip_table.add_row("Tip Header", tips[i]["header"])
                        current_tip_table.add_row("Tip Text", tips[i]["text"])
                        console.print(current_tip_table)

                        tip_response = Prompt.ask(
                            "\nDo you want to (a)ccept, (r)eject, or (s)kip this tip? (Default: s)",
                            choices=[tips_type.value for tips_type in TipsReviewType],
                            default=TipsReviewType.SKIP.value,
                            show_default=False,
                        ).strip()

                        print("")
                        if tip_response == TipsReviewType.ACCEPTED.value:
                            # persists the tip in the db as it is accepted
                            accept_tip(tips, tips_copy, i, unit)
                            print("[green]Tip Accepted[/green]")
                            # given the option to pay a bonus to the worker who wrote the tip
                            bonus = FloatPrompt.ask(
                                "\nHow much would you like to bonus the tip submitter? (Default: 0.0)",
                                show_default=False,
                                default=0.0,
                            )
                            if bonus > 0:
                                reason = Prompt.ask(
                                    "\nWhat reason would you like to give the worker for this tip? NOTE: This will be shared with the worker.(Default: Thank you for submitting a tip!)",
                                    default="Thank you for submitting a tip!",
                                    show_default=False,
                                )
                                worker_id = float(unit_data["worker_id"])
                                worker = Worker.get(db, worker_id)
                                if worker is not None:
                                    bonus_successfully_paid = worker.bonus_worker(
                                        bonus, reason, unit
                                    )
                                    if bonus_successfully_paid:
                                        print(
                                            "\n[green]Bonus Successfully Paid![/green]\n"
                                        )
                                    else:
                                        print(
                                            "\n[red]There was an error when paying out your bonus[/red]\n"
                                        )

                        elif tip_response == TipsReviewType.REJECTED.value:
                            remove_tip_from_metadata(tips, tips_copy, i, unit)
                            print("Tip Rejected\n")
                        elif tip_response == TipsReviewType.SKIP.value:
                            print("Tip Skipped\n")

    print("There are no more tips to review\n")


if __name__ == "__main__":
    main()



"""
Script that allows for feedback to be reviewed.
Feedback are collected by retrieving all units and then getting the agent of each unit.
The agent state has the feedback data.

When running the script you first have the option of
filtering out potential toxic feedback.
After that you have the ability to view reviewed or unreviewed feedback.

If you select reviewed feedback, then the script will print out all of the reviewed feedback.
If you select unreviewed feedback, then the script will print out each
unreviewed feedback and give you the option to mark the feedback as reviewed.
"""

import enum
from typing import Any, Dict, List
from mephisto.abstractions.databases.local_database import LocalMephistoDB
from mephisto.data_model.agent import Agent
from mephisto.data_model.unit import Unit
from mephisto.scripts.local_db.review_tips_for_task import get_index_of_value
from mephisto.tools.data_browser import DataBrowser as MephistoDataBrowser
from rich import print
from rich.markdown import Markdown
from rich.prompt import Prompt, IntPrompt
from mephisto.tools.scripts import print_out_task_names
from mephisto.utils.rich import console
from rich.table import Table
from rich import box

yes_response = set(["yes", "y", "YES", "Yes"])


class FeedbackReviewType(enum.Enum):
    YES = "y"
    NO = "n"
    REVIEWED = "r"
    UNREVIEWED = "u"


def set_feedback_as_reviewed(feedback: List, id: str, unit: Unit) -> None:
    """Sets the reviewed property of a feedback to true"""
    assigned_agent = unit.get_assigned_agent()
    feedback_ids = [feedback_obj["id"] for feedback_obj in feedback]
    index_to_modify = get_index_of_value(feedback_ids, id)
    if assigned_agent is not None:
        feedback[index_to_modify]["reviewed"] = True
        assigned_agent.state.update_metadata(
            property_name="feedback", property_value=feedback
        )


def print_out_reviewed_feedback_elements(
    filtered_feedback_list: List[Dict[str, Any]], agent: Agent
) -> None:
    if agent is not None:
        feedback_table = Table(
            "Id",
            "Question",
            "Text",
            "Toxicity",
            title="Reviewed Feedback For Agent: " + agent.get_agent_id(),
            box=box.ROUNDED,
            expand=True,
            show_lines=True,
        )
        filtered_feedback_list.sort(key=lambda x: x["question"])
        for feedback_obj in filtered_feedback_list:
            feedback_table.add_row(
                feedback_obj["id"],
                feedback_obj["question"],
                feedback_obj["text"],
                "[red]" + feedback_obj["toxicity"] + "[/red]"
                if float(feedback_obj["toxicity"]) > 0.5
                else feedback_obj["toxicity"],
            )
        console.print(feedback_table)


def print_out_unreviewed_feedback_elements(
    filtered_feedback_list: List[Dict[str, Any]],
    unit: Unit,
    feedback: List[Dict[str, Any]],
) -> None:
    """Prints out the desired feedback object properties from a filtered list"""
    i = 0
    agent = unit.get_assigned_agent()
    for feedback_obj in filtered_feedback_list:
        feedback_table = Table(
            "Property",
            "Value",
            title="Unreviewed Feedback {current_feedback} of {total_feedback} From Agent {agent_id}".format(
                current_feedback=i + 1,
                total_feedback=len(filtered_feedback_list),
                agent_id=agent.get_agent_id() if agent is not None else "-1",
            ),
            box=box.ROUNDED,
            expand=True,
            show_lines=True,
        )
        feedback_table.add_row("Id", feedback_obj["id"])
        feedback_table.add_row("Question", feedback_obj["question"])
        feedback_table.add_row("Text", feedback_obj["text"])
        feedback_table.add_row("Toxicity", feedback_obj["toxicity"])

        console.print(feedback_table)
        mark_feedback_as_reviewed = Prompt.ask(
            "\nDo you want to mark this feedback as reviewed? (Default: y)",
            choices=[FeedbackReviewType.YES.value, FeedbackReviewType.NO.value],
            default=FeedbackReviewType.YES.value,
            show_default=False,
        )
        if mark_feedback_as_reviewed == FeedbackReviewType.YES.value:
            set_feedback_as_reviewed(feedback, feedback_obj["id"], unit)
            print("\nMarked the feedback as reviewed!")

        elif mark_feedback_as_reviewed == FeedbackReviewType.NO.value:
            print("\nDid not mark the feedback as reviewed!")

        print("")
        i += 1


def main():
    db = LocalMephistoDB()
    mephisto_data_browser = MephistoDataBrowser(db)
    task_names = mephisto_data_browser.get_task_name_list()
    print_out_task_names("Feedback Review", task_names)
    task_name = Prompt.ask(
        "\nEnter the name of the task that you want to review the tips of",
        choices=task_names,
        show_choices=False,
    ).strip()
    print("")

    # Checking if there are units associated with the task name
    units = mephisto_data_browser.get_all_units_for_task_name(task_name)
    if len(units) == 0:
        print("[red]No units were received[/red]")
        quit()

    # Getting all questions
    questions = set({})
    for unit in units:
        if unit.agent_id is not None:
            unit_data = mephisto_data_browser.get_data_from_unit(unit)
            feedback = unit_data["feedback"]
            if feedback is not None:
                for current_feedback in feedback:
                    questions.add(current_feedback["question"])
    questions_list = list(questions)
    questions_list.sort()
    questions_text = """## Questions"""
    # Displaying all the questions alphabetically from the feedback component on the task webapp
    for i in range(len(questions_list)):
        questions_text += "\n" + str(i) + ". " + questions_list[i]
    questions_markdown = Markdown(questions_text)
    console.print(questions_markdown)
    # Making sure that the filter index is valid
    filter_by_question_index = IntPrompt.ask(
        '\nIf you want to filter feedback by a question, then enter the question number to filter on.\nIf you want to see feedback to all questions, enter "-1" (Default: -1)',
        choices=[str(i) for i in range(-1, len(questions_list))],
        default=-1,
        show_default=False,
    )

    if filter_by_question_index != -1:
        print(
            "\nYou chose to look at feedback from the following question: [bold]"
            + str(questions_list[filter_by_question_index])
            + "[/bold]\n"
        )

    # Allowing user to filter out toxic comments
    filter_toxic_comments = Prompt.ask(
        "Do you want to filter out toxic comments? (Default: n)",
        choices=[FeedbackReviewType.YES.value, FeedbackReviewType.NO.value],
        default=FeedbackReviewType.NO.value,
        show_default=False,
    )

    # Allowing user to see reviewed feedback or unreviewed feedback
    see_unreviewed_feedback = Prompt.ask(
        "Do you want to see (r)eviewed or (u)nreviewed feedback? (Default: u)",
        choices=[
            FeedbackReviewType.REVIEWED.value,
            FeedbackReviewType.UNREVIEWED.value,
        ],
        default=FeedbackReviewType.UNREVIEWED.value,
        show_default=False,
    )
    print("")

    for unit in units:
        if unit.agent_id is not None:
            unit_data = mephisto_data_browser.get_data_from_unit(unit)
            feedback = unit_data["feedback"]
            if feedback is not None:
                filtered_feedback = feedback.copy()
                if len(feedback) > 0:
                    # Firstly, filter by question if an index is specified
                    if filter_by_question_index != -1:
                        filtered_feedback = list(
                            filter(
                                lambda feedback_obj: feedback_obj["question"]
                                == questions_list[filter_by_question_index],
                                feedback,
                            )
                        )
                    if filter_toxic_comments == FeedbackReviewType.YES.value:
                        # Secondly, filter the question feedback for toxicity
                        filtered_feedback = list(
                            filter(
                                lambda feedback_obj: feedback_obj["toxicity"] is None
                                or float(feedback_obj["toxicity"]) < 0.5,
                                filtered_feedback,
                            )
                        )

                    if see_unreviewed_feedback == FeedbackReviewType.REVIEWED.value:
                        # Filter the toxicity feedback to get reviewed feedback
                        reviewed_feedback = list(
                            filter(
                                lambda feedback_obj: feedback_obj["reviewed"] == True,
                                filtered_feedback,
                            )
                        )
                        print_out_reviewed_feedback_elements(
                            filtered_feedback_list=reviewed_feedback,
                            agent=unit.get_assigned_agent(),
                        )
                    elif see_unreviewed_feedback == FeedbackReviewType.UNREVIEWED.value:
                        # Filter the toxicity feedback to get unreviewed feedback
                        un_reviewed_feedback = list(
                            filter(
                                lambda feedback_obj: feedback_obj["reviewed"] == False,
                                filtered_feedback,
                            )
                        )
                        print_out_unreviewed_feedback_elements(
                            filtered_feedback_list=un_reviewed_feedback,
                            unit=unit,
                            feedback=feedback,
                        )

    print(
        "[green]You went through all the {type_of_feedback} feedback{ending}![/green]\n".format(
            type_of_feedback="unreviewed"
            if see_unreviewed_feedback == FeedbackReviewType.UNREVIEWED.value
            else "reviewed",
            ending=" for this question" if filter_by_question_index != -1 else "",
        )
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import time
from mephisto.utils.metrics import (
    launch_grafana_server,
    launch_prometheus_server,
    get_dash_url,
)
from mephisto.utils.logger_core import set_mephisto_log_level


def launch_servers():
    """
    Launches a prometheus and grafana server instances and print the address as well as shutdown instructions
    """
    print("Launching servers")
    set_mephisto_log_level(level="info")
    if not launch_grafana_server():
        print("Issue launching grafana, see above")
        return
    if not launch_prometheus_server():
        print("Issue launching prometheus, see above")
        return
    print(f"Waiting for grafana server to come up.")
    time.sleep(3)
    dash_url = get_dash_url()
    print(f"Dashboard is now running, you can access it at {dash_url}")
    print(
        f"Once you're no longer using it, and no jobs need it anymore, you can shutdown with `shutdown_metrics.py`"
    )


if __name__ == "__main__":
    launch_servers()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin

from mephisto.utils.dirs import get_root_dir
from mephisto.operations.config_handler import DEFAULT_CONFIG_FOLDER
import os


class MephistoSearchPathPlugin(SearchPathPlugin):
    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        # Appends the search path for this plugin to the end of the search path
        profile_path = os.path.join(get_root_dir(), "hydra_configs")
        profile_path_user = os.path.join(DEFAULT_CONFIG_FOLDER, "hydra_configs")

        search_path.append(provider="mephisto-profiles", path=f"file://{profile_path}")
        search_path.append(
            provider="mephisto-profiles-user", path=f"file://{profile_path_user}"
        )



from mephisto.abstractions.databases.local_database import LocalMephistoDB

"""
This script should not be used locally.
It is only used in a GitHub action as it does not automatically expire units.
"""


def main():
    task_name = "react-static-task-with-tips"
    db = LocalMephistoDB()
    tasks = db.find_tasks(task_name=task_name)
    assert len(tasks) >= 1, f"No task found under name {task_name}"
    task_runs = db.find_task_runs(task_id=tasks[0].db_id)
    for task_run in task_runs:
        assignments = task_run.get_assignments()
        for assignment in assignments:
            found_units = assignment.get_units()
            for unit in found_units:
                unit.set_db_status("expired")


if __name__ == "__main__":
    main()



from mdutils.mdutils import MdUtils
from mephisto.client.cli_commands import get_wut_arguments
from mephisto.operations.registry import (
    get_valid_provider_types,
)
from mephisto.scripts.local_db.gh_actions.auto_generate_blueprint import (
    create_blueprint_info,
)


def main():
    requester_file = MdUtils(
        file_name="../../../../docs/web/docs/reference/requesters.md",
    )
    requester_file.new_header(level=1, title="Requesters")
    requester_file.new_paragraph(
        "Requesters are Mephisto's wrapper around an identity for a CrowdProvider, usually storing the credentials for an account to launch tasks from."
    )
    requester_file.new_paragraph(
        "Use `mephisto requesters` to see registered requesters, and `mephisto register <requester type>` to register."
    )
    valid_requester_types = get_valid_provider_types()
    for requester_type in valid_requester_types:
        requester_file.new_header(level=2, title=requester_type.replace("_", " "))
        args = get_wut_arguments(
            ("requester={requester_name}".format(requester_name=requester_type),)
        )
        arg_dict = args[0]
        create_blueprint_info(requester_file, arg_dict)

    requester_file.create_md_file()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


from typing import List
from rich import print
from mephisto.client.cli_commands import get_wut_arguments
from mephisto.operations.registry import get_valid_provider_types
from mephisto.utils.rich import console, create_table
import rich_click as click  # type: ignore
import os
from rich_click import RichCommand, RichGroup
from rich.markdown import Markdown
import mephisto.scripts.local_db.review_tips_for_task as review_tips_local_db
import mephisto.scripts.local_db.remove_accepted_tip as remove_accepted_tip_local_db
import mephisto.scripts.local_db.review_feedback_for_task as review_feedback_local_db
import mephisto.scripts.local_db.load_data_to_mephisto_db as load_data_local_db
import mephisto.scripts.local_db.clear_worker_onboarding as clear_worker_onboarding_local_db
import mephisto.scripts.heroku.initialize_heroku as initialize_heroku
import mephisto.scripts.metrics.view_metrics as view_metrics
import mephisto.scripts.metrics.shutdown_metrics as shutdown_metrics
import mephisto.scripts.mturk.cleanup as cleanup_mturk
import mephisto.scripts.mturk.identify_broken_units as identify_broken_units_mturk
import mephisto.scripts.mturk.launch_makeup_hits as launch_makeup_hits_mturk
import mephisto.scripts.mturk.print_outstanding_hit_status as print_outstanding_hit_status_mturk
import mephisto.scripts.mturk.print_outstanding_hit_status as soft_block_workers_by_mturk_id_mturk


@click.group(cls=RichGroup)
def cli():
    """[deep_sky_blue4]Bring your research ideas to life
    with powerful crowdsourcing tooling[/]
    """


click.rich_click.USE_RICH_MARKUP = True
click.rich_click.SHOW_ARGUMENTS = True
click.rich_click.ERRORS_SUGGESTION = (
    "\nTry running the '--help' flag for more information."
)
click.rich_click.ERRORS_EPILOGUE = (
    "To find out more, visit https://mephisto.ai/docs/guides/quickstart/\n"
)


@cli.command("web", cls=RichCommand)
def web():
    """Launch a local webserver with the Mephisto UI"""
    from mephisto.client.full.server import get_app

    get_app().run(debug=False)


@cli.command("config", cls=RichCommand)
@click.argument("identifier", type=(str), default=None, required=False)
@click.argument("value", type=(str), default=None, required=False)
def config(identifier, value):
    from mephisto.operations.config_handler import (
        get_config_arg,
        add_config_arg,
        get_raw_config,
        DEFAULT_CONFIG_FILE,
    )

    if identifier is None and value is None:
        # If no args, show full config:
        click.echo(f"{DEFAULT_CONFIG_FILE}:\n")
        click.echo(get_raw_config())
        return

    if "." not in identifier:
        raise click.BadParameter(
            f"Identifier must be of format: <section>.<key>\nYou passed in: {identifier}"
        )
    [section, key] = identifier.split(".")

    if value is None:
        # Read mode:
        click.echo(get_config_arg(section, key))
    else:
        # Write mode:
        add_config_arg(section, key, value)
        print(f"[green]{identifier} succesfully updated to: {value}[/green]")


@cli.command(
    "review",
    cls=RichCommand,
)
@click.argument(
    "review_app_dir",
    type=click.Path(exists=True),
    default=os.path.join(os.path.dirname(__file__), "review/default-ui"),
)
@click.option("-p", "--port", type=(int), default=5000)
@click.option("-o", "--output", type=(str), default="")
@click.option("-a", "--assets", "assets_dir", type=(str), default=None)
@click.option("--stdout", "output_method", flag_value="stdout")
@click.option("--file", "output_method", flag_value="file", default=True)
@click.option("--csv-headers/--no-csv-headers", default=False)
@click.option("--json/--csv", default=False)
@click.option("--db", "database_task_name", type=(str), default=None)
@click.option("--all/--one-by-one", "all_data", default=False)
@click.option("-d", "--debug", type=(bool), default=False)
def review(
    review_app_dir,
    port,
    output,
    output_method,
    csv_headers,
    json,
    database_task_name,
    all_data,
    debug,
    assets_dir,
):
    """Launch a local review UI server. Reads in rows froms stdin and outputs to either a file or stdout."""
    from mephisto.client.review.review_server import run

    if output == "" and output_method == "file":
        raise click.UsageError(
            "You must specify an output file via --output=<filename>, unless the --stdout flag is set."
        )
    if database_task_name is not None:
        from mephisto.abstractions.databases.local_database import LocalMephistoDB
        from mephisto.tools.data_browser import DataBrowser as MephistoDataBrowser

        db = LocalMephistoDB()
        mephisto_data_browser = MephistoDataBrowser(db=db)
        name_list = mephisto_data_browser.get_task_name_list()
        if database_task_name not in name_list:
            raise click.BadParameter(
                f'The task name "{database_task_name}" did not exist in MephistoDB.\n\nPerhaps you meant one of these? {", ".join(name_list)}\n\nFlag usage: mephisto review --db [task_name]\n'
            )

    run(
        review_app_dir,
        port,
        output,
        csv_headers,
        json,
        database_task_name,
        all_data,
        debug,
        assets_dir,
    )


@cli.command("check", cls=RichCommand)
def check():
    """Checks that mephisto is setup correctly"""
    from mephisto.abstractions.databases.local_database import LocalMephistoDB
    from mephisto.utils.testing import get_mock_requester

    try:
        db = LocalMephistoDB()
        get_mock_requester(db)
    except Exception as e:
        print("\n[red]Something went wrong.[/red]")
        click.echo(e)
        return
    print("\n[green]Mephisto seems to be set up correctly.[/green]\n")


@cli.command("requesters", cls=RichCommand)
def list_requesters():
    """Lists all registered requesters"""
    from mephisto.abstractions.databases.local_database import LocalMephistoDB

    db = LocalMephistoDB()
    requesters = db.find_requesters()
    dict_requesters = [r.to_dict() for r in requesters]
    if len(dict_requesters) > 0:
        requester_keys = list(dict_requesters[0].keys())
        requester_table = create_table(requester_keys, "\n\n Requesters")
        for requester in dict_requesters:
            requester_vals = list(requester.values())
            requester_vals = [str(x) for x in requester_vals]
            requester_table.add_row(*requester_vals)
        console.print(requester_table)
    else:
        print("[red]No requesters found[/red]")


@cli.command(
    "register", cls=RichCommand, context_settings={"ignore_unknown_options": True}
)
@click.argument("args", nargs=-1)
def register_provider(args):
    """Register a requester with a crowd provider"""
    if len(args) == 0:
        print(
            "\n[red]Usage: mephisto register <provider_type> arg1=value arg2=value[/red]"
        )
        print("\n[b]Valid Providers[/b]")
        provider_text = """"""
        for provider in get_valid_provider_types():
            provider_text += "\n* " + provider
        provider_text_markdown = Markdown(provider_text)
        console.print(provider_text_markdown)
        print("")
        return

    from mephisto.abstractions.databases.local_database import LocalMephistoDB
    from mephisto.operations.registry import get_crowd_provider_from_type
    from mephisto.operations.hydra_config import (
        parse_arg_dict,
        get_extra_argument_dicts,
    )

    provider_type, requester_args = args[0], args[1:]
    args_dict = dict(arg.split("=", 1) for arg in requester_args)

    crowd_provider = get_crowd_provider_from_type(provider_type)
    RequesterClass = crowd_provider.RequesterClass

    if len(requester_args) == 0:
        params = get_extra_argument_dicts(RequesterClass)
        for param in params:
            click.echo("\n" + param["desc"])
            param_keys = list(param["args"].keys())
            if len(param_keys) > 0:
                first_arg_key = param_keys[0]
                requester_headers = list(param["args"][first_arg_key].keys())
                requester_table = create_table(requester_headers, "[b]Arguments[/b]")
                for arg in param["args"]:
                    arg_values = list(param["args"][arg].values())
                    arg_values = [str(x) for x in arg_values]
                    requester_table.add_row(*arg_values)
                console.print(requester_table)
            else:
                print("[red]Requester has no args[/red]")
        return

    try:
        parsed_options = parse_arg_dict(RequesterClass, args_dict)
    except Exception as e:
        click.echo(str(e))

    if parsed_options.name is None:
        print("[red]No name was specified for the requester.[/red]")

    db = LocalMephistoDB()
    requesters = db.find_requesters(requester_name=parsed_options.name)
    if len(requesters) == 0:
        requester = RequesterClass.new(db, parsed_options.name)
    else:
        requester = requesters[0]
    try:
        requester.register(parsed_options)
        print("[green]Registered successfully.[/green]")
    except Exception as e:
        click.echo(str(e))


@cli.command("wut", cls=RichCommand, context_settings={"ignore_unknown_options": True})
@click.argument("args", nargs=-1)
def run_wut(args):
    """Discover the configuration arguments for different abstractions"""
    get_wut_arguments(args)


@cli.command(
    "scripts", cls=RichCommand, context_settings={"ignore_unknown_options": True}
)
@click.argument("script_type", required=False, nargs=1)
@click.argument("script_name", required=False, nargs=1)
def run_script(script_type, script_name):
    """Run one of the many mephisto scripts."""

    def print_non_markdown_list(items: List[str]):
        res = ""
        for item in items:
            res += "\n  * " + item
        return res

    VALID_SCRIPT_TYPES = ["local_db", "heroku", "metrics", "mturk"]
    if script_type is None or script_type.strip() not in VALID_SCRIPT_TYPES:
        print("")
        raise click.UsageError(
            "You must specify a valid script_type from below. \n\nValid script types are:"
            + print_non_markdown_list(VALID_SCRIPT_TYPES)
        )
    script_type = script_type.strip()
    LOCAL_DB_VALID_SCRIPTS_NAMES = [
        "review_tips",
        "remove_tip",
        "review_feedback",
        "load_data",
        "clear_worker_onboarding",
    ]
    HEROKU_VALID_SCRIPTS_NAMES = ["initialize"]
    METRICS_VALID_SCRIPTS_NAMES = ["view", "shutdown"]
    MTURK_VALID_SCRIPTS_NAMES = [
        "cleanup",
        "identify_broken_units",
        "launch_makeup_hits",
        "print_outstanding_hit_status",
        "soft_block_workers_by_mturk_id",
    ]
    script_type_to_scripts_data = {
        "local_db": {
            "valid_script_names": LOCAL_DB_VALID_SCRIPTS_NAMES,
            "scripts": {
                LOCAL_DB_VALID_SCRIPTS_NAMES[0]: review_tips_local_db.main,
                LOCAL_DB_VALID_SCRIPTS_NAMES[1]: remove_accepted_tip_local_db.main,
                LOCAL_DB_VALID_SCRIPTS_NAMES[2]: review_feedback_local_db.main,
                LOCAL_DB_VALID_SCRIPTS_NAMES[3]: load_data_local_db.main,
                LOCAL_DB_VALID_SCRIPTS_NAMES[4]: clear_worker_onboarding_local_db.main,
            },
        },
        "heroku": {
            "valid_script_names": HEROKU_VALID_SCRIPTS_NAMES,
            "scripts": {HEROKU_VALID_SCRIPTS_NAMES[0]: initialize_heroku.main},
        },
        "metrics": {
            "valid_script_names": METRICS_VALID_SCRIPTS_NAMES,
            "scripts": {
                METRICS_VALID_SCRIPTS_NAMES[0]: view_metrics.launch_servers,
                METRICS_VALID_SCRIPTS_NAMES[1]: shutdown_metrics.shutdown_servers,
            },
        },
        "mturk": {
            "valid_script_names": MTURK_VALID_SCRIPTS_NAMES,
            "scripts": {
                MTURK_VALID_SCRIPTS_NAMES[0]: cleanup_mturk.main,
                MTURK_VALID_SCRIPTS_NAMES[1]: identify_broken_units_mturk.main,
                MTURK_VALID_SCRIPTS_NAMES[2]: launch_makeup_hits_mturk.main,
                MTURK_VALID_SCRIPTS_NAMES[3]: print_outstanding_hit_status_mturk.main,
                MTURK_VALID_SCRIPTS_NAMES[4]: soft_block_workers_by_mturk_id_mturk.main,
            },
        },
    }

    if script_name is None or (
        script_name
        not in script_type_to_scripts_data[script_type]["valid_script_names"]
    ):
        print("")
        raise click.UsageError(
            "You must specify a valid script_name from below. \n\nValid script names are:"
            + print_non_markdown_list(
                script_type_to_scripts_data[script_type]["valid_script_names"]
            )
        )
    # runs the script
    script_type_to_scripts_data[script_type]["scripts"][script_name]()


@cli.command(
    "metrics", cls=RichCommand, context_settings={"ignore_unknown_options": True}
)
@click.argument("args", nargs=-1)
def metrics_cli(args):
    from mephisto.utils.metrics import (
        launch_servers_and_wait,
        metrics_are_installed,
        run_install_script,
        METRICS_DIR,
        shutdown_prometheus_server,
        shutdown_grafana_server,
    )

    if len(args) == 0 or args[0] not in ["install", "view", "cleanup"]:
        print("\n[red]Usage: mephisto metrics <install|view|cleanup>[/red]")
        metrics_table = create_table(["Property", "Value"], "Metrics Arguments")
        metrics_table.add_row(
            "install", f"Installs Prometheus and Grafana to {METRICS_DIR}"
        )
        metrics_table.add_row(
            "view",
            "Launches a Prometheus and Grafana server, and shuts down on exit",
        )
        metrics_table.add_row(
            "cleanup",
            "Shuts down Prometheus and Grafana resources that may have persisted",
        )
        console.print(metrics_table)
        return
    command = args[0]
    if command == "install":
        if metrics_are_installed():
            click.echo(f"Metrics are already installed! See {METRICS_DIR}")
            return
        run_install_script()
    elif command == "view":
        if not metrics_are_installed():
            click.echo(
                f"Metrics aren't installed! Use `mephisto metrics install` first."
            )
            return
        click.echo(f"Servers launching - use ctrl-C to shutdown")
        launch_servers_and_wait()
    else:  # command == 'cleanup':
        if not metrics_are_installed():
            click.echo(
                f"Metrics aren't installed! Use `mephisto metrics install` first."
            )
            return
        click.echo(f"Cleaning up existing servers if they exist")
        shutdown_prometheus_server()
        shutdown_grafana_server()


if __name__ == "__main__":
    cli()



from mdutils.mdutils import MdUtils
from mephisto.client.cli_commands import get_wut_arguments
from mephisto.operations.registry import (
    get_valid_provider_types,
)
from mephisto.scripts.local_db.gh_actions.auto_generate_blueprint import (
    create_blueprint_info,
)


def main():
    provider_file = MdUtils(
        file_name="../../../../docs/web/docs/reference/providers.md",
    )
    provider_file.new_header(level=1, title="Providers")
    provider_file.new_paragraph(
        "Crowd providers standardize access to external crowd workers, by wrapping external API communication through a standardized interface."
    )
    valid_provider_types = get_valid_provider_types()
    for provider_type in valid_provider_types:
        provider_file.new_header(level=2, title=provider_type.replace("_", " "))
        args = get_wut_arguments(
            ("provider={provider_name}".format(provider_name=provider_type),)
        )
        arg_dict = args[0]
        create_blueprint_info(provider_file, arg_dict)

    provider_file.create_md_file()


if __name__ == "__main__":
    main()



from mephisto.utils.rich import console, print_out_valid_options, create_table
import rich_click as click  # type: ignore
from rich import print


def get_wut_arguments(args):
    """Display information about hydra config properties"""
    from mephisto.operations.registry import (
        get_blueprint_from_type,
        get_crowd_provider_from_type,
        get_architect_from_type,
        get_valid_blueprint_types,
        get_valid_provider_types,
        get_valid_architect_types,
    )

    if len(args) == 0:
        print(
            "\n[red]Usage: mephisto wut <abstraction>[=<type>] [...specific args to check][/red]"
        )
        abstractions_table = create_table(
            ["Abstraction", "Description"], "\n\n[b]Abstractions[/b]"
        )
        abstractions_table.add_row(
            "blueprint",
            f"The blueprint contains all of the related code required to set up a task run. \nValid blueprints types are [b]{get_valid_blueprint_types()}[/b]",
        )
        abstractions_table.add_row(
            "architect",
            f"Architect's contain the logic surrounding deploying a server that workers will be able to access. \nValid architects types are [b]{get_valid_architect_types()}[/b]",
        )
        abstractions_table.add_row(
            "requester",
            f"The requester is an account for a crowd provider. Requesters are used as the identity that launches the task. \nValid requester types types are [b]{get_valid_provider_types()}[/b]. \n"
            "\nUse `mephisto requesters` to see registered requesters, and `mephisto register <requester type>` to register.",
        )
        abstractions_table.add_row(
            "provider",
            f"The crowd provider is responsible for standardizing Mephisto's interaction with external crowds. \nValid provider types are [b]{get_valid_provider_types()}[/b]",
        )
        console.print(abstractions_table)
        return

    VALID_ABSTRACTIONS = ["blueprint", "architect", "requester", "provider", "task"]

    from mephisto.operations.hydra_config import (
        get_extra_argument_dicts,
        get_task_state_dicts,
    )

    VALID_ABSTRACTIONS = ["blueprint", "architect", "requester", "provider", "task"]

    abstraction_equal_split = args[0].split("=", 1)
    abstraction = abstraction_equal_split[0]

    if abstraction not in VALID_ABSTRACTIONS:
        print(
            f"[red]Given abstraction {abstraction} not in valid abstractions {VALID_ABSTRACTIONS}][/red]"
        )
        return

    if abstraction == "task":
        from mephisto.data_model.task_run import TaskRun

        target_class = TaskRun
    else:
        if len(abstraction_equal_split) == 1:
            # querying about the general abstraction
            if abstraction == "blueprint":
                click.echo("The blueprint determines the task content.\n")
                valid_blueprints_text = """**Valid blueprints are:**"""
                print_out_valid_options(
                    valid_blueprints_text, get_valid_blueprint_types()
                )
                return
            elif abstraction == "architect":
                click.echo(
                    "The architect determines the server where a task is hosted.\n"
                )
                valid_architect_text = """**Valid architects are:**"""
                print_out_valid_options(
                    valid_architect_text, get_valid_architect_types()
                )
                return
            elif abstraction == "requester":
                click.echo(
                    f"The requester is an account for a crowd provider. \n"
                    "Use `mephisto requesters` to see registered requesters, and `mephisto register <requester type>` to register.\n"
                )
                valid_requester_text = """**Valid requesters are:**"""
                print_out_valid_options(
                    valid_requester_text, get_valid_provider_types()
                )
                return
            elif abstraction == "provider":
                click.echo(
                    "The crowd provider determines the source of the crowd workers.\n"
                )
                valid_provider_text = """**Valid providers are:**"""
                print_out_valid_options(valid_provider_text, get_valid_provider_types())
                return

        # There's a specific abstraction to check
        abstract_value = abstraction_equal_split[1]
        target_class = None
        valid = None
        if abstraction == "blueprint":
            try:
                target_class = get_blueprint_from_type(abstract_value)
            except:
                valid = get_valid_blueprint_types()
        elif abstraction == "architect":
            try:
                target_class = get_architect_from_type(abstract_value)
            except:
                valid = get_valid_architect_types()
        elif abstraction == "provider":
            try:
                target_class = get_crowd_provider_from_type(abstract_value)
            except:
                valid = get_valid_provider_types()
        elif abstraction == "requester":
            try:
                target_class = get_crowd_provider_from_type(
                    abstract_value
                ).RequesterClass
            except:
                valid = get_valid_provider_types()
        if valid is not None:
            print(f"\n[b]The valid types for {abstraction} are:[/b]")
            valid_options_text = """"""
            print_out_valid_options(valid_options_text, valid)
            print(f"[red]'{abstract_value}' not found[/red]\n")
            return

    # These are values that do not convert to a string well or are incorrect, so they need to be hardcoded
    argument_overrides = {
        "tips_location": ("default", "path_to_task/assets/tips.csv"),
        "heroku_config_args": ("default", "{}"),
        "subdomain": ("default", "The task name defined in your task's hydra config"),
    }

    arg_dict = get_extra_argument_dicts(target_class)[0]
    click.echo(arg_dict["desc"])
    checking_args = arg_dict["args"]
    if len(args) > 1:
        checking_args = {k: v for k, v in checking_args.items() if k in args[1:]}
    checking_args_keys = list(checking_args.keys())
    if len(checking_args_keys) > 0:
        first_arg = checking_args_keys[0]
        first_arg_keys = list(checking_args[first_arg].keys())
        args_table = create_table(
            first_arg_keys,
            "\n[b]{abstraction} Arguments[/b]".format(
                abstraction=abstraction.capitalize()
            ),
        )
        for arg in checking_args:
            if arg in argument_overrides:
                checking_args[arg][argument_overrides[arg][0]] = argument_overrides[
                    arg
                ][1]
            arg_values = list(checking_args[arg].values())
            arg_values = [str(x) for x in arg_values]
            args_table.add_row(*arg_values)

        console.print(args_table)
        if abstraction != "blueprint":
            return [arg_dict]
    if abstraction == "blueprint":
        state_args = get_task_state_dicts(target_class)[0]["args"]
        if len(args) > 1:
            state_args = {k: v for k, v in state_args.items() if k in args[1:]}
        state_args_keys = list(state_args.keys())
        if len(state_args_keys) > 0:
            click.echo(
                f"\n\nAdditional SharedTaskState args from {target_class.SharedStateClass.__name__}, which may be configured in your run script"
            )
            first_state_arg = state_args_keys[0]
            first_arg_keys = list(state_args[first_state_arg].keys())

            state_args_table = create_table(
                first_arg_keys, "\n[b]Additional Shared TaskState args[/b]"
            )

            for arg in state_args:
                arg_values = list(state_args[arg].values())
                arg_values = [str(x) for x in arg_values]
                state_args_table.add_row(*arg_values)
            console.print(state_args_table)
            return [arg_dict, state_args]



from typing import List
from rich.console import Console
from rich.table import Table
from rich import box
from rich.markdown import Markdown

console = Console()


def create_table(headers: List[str], title: str) -> Table:
    return Table(*headers, title=title, box=box.ROUNDED, expand=True, show_lines=True)


def print_out_valid_options(markdown_text: str, valid_options: List[str]) -> None:
    for valid_option in valid_options:
        markdown_text += "\n* " + valid_option
    console.print(Markdown(markdown_text))
    console.print("")



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import time
from typing import List, Dict, Optional, Any, TYPE_CHECKING
from mephisto.abstractions.blueprint import AgentState
import os.path

if TYPE_CHECKING:
    from mephisto.data_model.agent import Agent
    from mephisto.data_model.packet import Packet

from mephisto.utils.logger_core import get_logger

logger = get_logger(name=__name__)

DATA_FILE = "agent_data.json"


class StaticAgentState(AgentState):
    """
    Agent state for static tasks.
    """

    def _get_empty_state(self) -> Dict[str, Optional[Dict[str, Any]]]:
        return {
            "inputs": None,
            "outputs": None,
        }

    def _set_init_state(self, data: Any):
        """Set the initial state for this agent"""
        self.state["inputs"] = data

    def get_init_state(self) -> Optional[Dict[str, Any]]:

        """
        Return the initial state for this agent,
        None if no such state exists
        """
        if self.state["inputs"] is None:
            return None
        return self.state["inputs"].copy()

    def _load_data(self) -> None:
        """Load data for this agent from disk"""
        data_dir = self.agent.get_data_dir()
        data_path = os.path.join(data_dir, DATA_FILE)
        if self.agent.db.key_exists(data_path):
            self.state = self.agent.db.read_dict(data_path)
            # Old compatibility with saved times
            if "times" in self.state:
                assert isinstance(self.state["times"], dict)
                self.metadata.task_start = self.state["times"]["task_start"]
                self.metadata.task_end = self.state["times"]["task_end"]
        else:
            self.state = self._get_empty_state()

    def get_data(self) -> Dict[str, Any]:
        """Return dict of this agent's state"""
        return self.state.copy()

    def _save_data(self) -> None:
        """Save static agent data to disk"""
        data_dir = self.agent.get_data_dir()
        out_filename = os.path.join(data_dir, DATA_FILE)
        self.agent.db.write_dict(out_filename, self.state)
        logger.info(f"SAVED_DATA_TO_DISC at {out_filename}")

    def update_data(self, live_update: Dict[str, Any]) -> None:
        """
        Process the incoming data packet, and handle updating the state
        """
        raise Exception("Static tasks should only have final act, but got live update")

    def _update_submit(self, submission_data: Dict[str, Any]) -> None:
        """Move the submitted output to the local dict"""
        outputs: Dict[str, Any]
        assert isinstance(submission_data, dict), (
            "Static tasks must get dict results. Ensure you are passing an object to "
            f"your frontend task's `handleSubmit` method. Got {submission_data}"
        )
        output_files = submission_data.get("files")
        if output_files is not None:
            submission_data["files"] = [f["filename"] for f in submission_data["files"]]
        self.state["outputs"] = submission_data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hydra_plugins',
 'mephisto',
 'mephisto.abstractions',
 'mephisto.abstractions._subcomponents',
 'mephisto.abstractions.architects',
 'mephisto.abstractions.architects.channels',
 'mephisto.abstractions.architects.ec2',
 'mephisto.abstractions.architects.ec2.fallback_server',
 'mephisto.abstractions.architects.router',
 'mephisto.abstractions.architects.router.flask',
 'mephisto.abstractions.blueprints',
 'mephisto.abstractions.blueprints.abstract',
 'mephisto.abstractions.blueprints.abstract.static_task',
 'mephisto.abstractions.blueprints.mixins',
 'mephisto.abstractions.blueprints.mock',
 'mephisto.abstractions.blueprints.parlai_chat',
 'mephisto.abstractions.blueprints.remote_procedure',
 'mephisto.abstractions.blueprints.static_html_task',
 'mephisto.abstractions.blueprints.static_react_task',
 'mephisto.abstractions.databases',
 'mephisto.abstractions.providers',
 'mephisto.abstractions.providers.mock',
 'mephisto.abstractions.providers.mturk',
 'mephisto.abstractions.providers.mturk.utils',
 'mephisto.abstractions.providers.mturk_sandbox',
 'mephisto.abstractions.test',
 'mephisto.client',
 'mephisto.client.full',
 'mephisto.client.review',
 'mephisto.data_model',
 'mephisto.data_model.constants',
 'mephisto.operations',
 'mephisto.scripts',
 'mephisto.scripts.heroku',
 'mephisto.scripts.local_db',
 'mephisto.scripts.local_db.gh_actions',
 'mephisto.scripts.metrics',
 'mephisto.scripts.mturk',
 'mephisto.tools',
 'mephisto.utils']

package_data = \
{'': ['*'],
 'mephisto.abstractions.architects.ec2': ['keypairs/*',
                                          'run_scripts/flask/*',
                                          'run_scripts/node/*',
                                          'servers/*'],
 'mephisto.abstractions.architects.ec2.fallback_server': ['access_logs/*',
                                                          'scripts/*',
                                                          'templates/*'],
 'mephisto.abstractions.architects.router': ['node/*',
                                             'node/static/*',
                                             'node/uploads/*'],
 'mephisto.abstractions.architects.router.flask': ['static/*'],
 'mephisto.abstractions.blueprints.parlai_chat': ['webapp/*',
                                                  'webapp/src/*',
                                                  'webapp/src/static/*'],
 'mephisto.abstractions.blueprints.static_html_task': ['source/*',
                                                       'source/cypress/e2e/*',
                                                       'source/cypress/fixtures/*',
                                                       'source/cypress/support/*',
                                                       'source/dev/*'],
 'mephisto.client.full': ['webapp/*',
                          'webapp/public/*',
                          'webapp/src/*',
                          'webapp/src/lib/*',
                          'webapp/src/widgets/*',
                          'webapp/src/widgets/components/*'],
 'mephisto.client.review': ['default-ui/*',
                            'default-ui/static/css/*',
                            'default-ui/static/js/*',
                            'default-ui/static/media/*'],
 'mephisto.scripts.metrics': ['resources/*']}

install_requires = \
['boto3>=1.17,<2.0',
 'botocore>=1.20.95,<2.0.0',
 'click>=7.0,<8.0',
 'click_default_group>=1.2,<2.0',
 'detoxify>=0.5.0,<0.6.0',
 'flask>=1.1,<2.0',
 'gevent-websocket>=0.10.1,<0.11.0',
 'hydra-core>=1.2.0,<2.0.0',
 'markupsafe==2.0.1',
 'prometheus-client>=0.13.1,<0.14.0',
 'requests>=2.22,<3.0',
 'rich-click>=1.5.1,<2.0.0',
 'rich>=12.5.1,<13.0.0',
 'sh>=1.12,<2.0',
 'tabulate>=0.8.7,<0.9.0',
 'tornado>=6.0,<7.0',
 'tqdm>=4.50.2,<5.0.0',
 'websockets>=10.1,<11.0',
 'werkzeug>=1.0.1,<2.0.0',
 'xmltodict>=0.12.0,<0.13.0']

extras_require = \
{':extra == "parlai"': ['pyyaml>=5.4.0,<6.0.0',
                        'parlai>=1.7.0,<2.0.0',
                        'torch>=1.4.0,<2.0.0']}

entry_points = \
{'console_scripts': ['mephisto = mephisto.client.cli:cli']}

setup_kwargs = {
    'name': 'mephisto',
    'version': '1.1.1',
    'description': 'Crowdsourcing made simpler.',
    'long_description': '<p align="center">\n <h1>\n  <img width="250px" src="docs/web/static/img/logo.svg" alt="Mephisto" />\n </h1>\n</p>\n\n[![code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![code style: prettier](https://img.shields.io/badge/code_style-prettier-ff69b4.svg)](https://github.com/prettier/prettier)\n\nMephisto makes crowdsourcing easier.\n\nWe provide a platform for launching, monitoring, and reviewing your crowdsourcing tasks. Tasks made on Mephisto can easily be open sourced as part of other projects. Like the [chess-playing automaton](https://en.wikipedia.org/wiki/Mephisto_(automaton)) we\'ve adopted the name from, Mephisto hides the complexity of projects that need human interaction or input.\n\nYou can find complete details about the project on our [docs website](https://mephisto.ai).\n\n## Quickstart\n\n[Get started in 10 minutes](https://mephisto.ai/docs/guides/quickstart)\n\n## Changelog\n\n## Want to help?\n\nCheck out our [guidelines for contributing](https://github.com/facebookresearch/Mephisto/blob/main/CONTRIBUTING.md) and then take a look at some of our tagged issues: [good first issue](https://github.com/facebookresearch/Mephisto/labels/good%20first%20issue), [help wanted](https://github.com/facebookresearch/Mephisto/labels/help%20wanted).\n\nFor library authors, you may also find the [how to contribute](https://mephisto.ai/docs/guides/how_to_contribute/getting_started) documentation helpful.\n\n\n## License\nMephisto is MIT licensed. See the LICENSE file for details.\n',
    'author': 'Jack Urbanek',
    'author_email': 'jju@fb.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://mephisto.ai/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from mdutils.mdutils import MdUtils
from mephisto.client.cli_commands import get_wut_arguments
from mephisto.operations.registry import get_valid_architect_types
from mephisto.scripts.local_db.gh_actions.auto_generate_blueprint import (
    create_blueprint_info,
)


def main():
    architect_file = MdUtils(
        file_name="../../../../docs/web/docs/reference/architects.md",
    )
    architect_file.new_header(level=1, title="Architects")
    architect_file.new_paragraph(
        "Architects contain the logic surrounding deploying a server that workers will be able to access."
    )
    valid_architect_types = get_valid_architect_types()
    for architect_type in valid_architect_types:
        architect_file.new_header(level=2, title=architect_type.replace("_", " "))
        args = get_wut_arguments(
            ("architect={architect_name}".format(architect_name=architect_type),)
        )
        arg_dict = args[0]
        create_blueprint_info(architect_file, arg_dict)

    architect_file.create_md_file()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from mephisto.utils.metrics import shutdown_grafana_server, shutdown_prometheus_server
from mephisto.utils.logger_core import set_mephisto_log_level


def shutdown_servers():
    """
    Launches a prometheus and grafana server instances and print the address as well as shutdown instructions
    """
    print("Using stored PIDs to shutdown metrics servers")
    set_mephisto_log_level(level="info")
    shutdown_prometheus_server()
    shutdown_grafana_server()
    print("Servers shut down!")


if __name__ == "__main__":
    shutdown_servers()



from mephisto.abstractions.blueprints.mixins.onboarding_required import (
    OnboardingRequired,
)
from mephisto.abstractions.databases.local_database import LocalMephistoDB
from mephisto.data_model.worker import Worker
from rich import print
from rich.prompt import Prompt

"""
In this script we are asking for a worker_id, but we are actually using it as a worker_name.
In the database a worker_id is a number like 1 or 2, while a worker_name is like "x_sandbox".
However, when looking at an url like: http://localhost:3000/?worker_id=x&assignment_id=45
the worker_id is "x".

As the user is more likely to think of the worker_id as "x" instead of 1, we
are calling the worker_name, the worker_id.

In the url example above, this script would first check if there are any workers with the name x.
If there is one, then that worker's onboarding qualification will be revoked. If there are
more than one worker with that name the script aborts.

If there are no workers with that name, it then tries the name "x_sandbox". The _sandbox
affix is commonly attached to worker names in the backend, so that is why "x_sandbox"
is also tried.
"""


def main():
    db = LocalMephistoDB()
    worker_id = str(Prompt.ask("What is the worker's id:", default="x"))

    desired_worker = db.find_workers(worker_name=worker_id)

    if len(desired_worker) == 0:
        worker_id_with_sandbox = worker_id + "_sandbox"
        sandbox_workers = db.find_workers(worker_name=worker_id_with_sandbox)

        if len(sandbox_workers) == 0:
            print("\n[red]There are no workers with that name[/red]\n")
            quit()

        elif len(sandbox_workers) > 1:
            print("\n[red] There is more that one worker with that name[/red]\n")
            quit()

        clear_onboarding(sandbox_workers[0])
        quit()

    elif len(desired_worker > 1):
        print("\n[red]There is more than one worker with that name[/red]\n")
        quit()

    clear_onboarding(desired_worker[0])


def clear_onboarding(desired_worker: Worker):
    qualification_name = Prompt.ask(
        "What is the qualification name:", default="test-react-static-qualification"
    )

    OnboardingRequired.clear_onboarding(
        worker=desired_worker, qualification_name=qualification_name
    )
    print("\n[green]Cleared that onboarding qualification[/green]\n")


if __name__ == "__main__":
    main()
